@@ -1,13 +1,13 @@
 import json
-import random
 import re
-import string
 import os
 
 import pypandoc
 
 from tqdm import tqdm
 
+from examtool.api.utils import rand_id
+
 VERSION = 2  # increment when backward-incompatible changes are made
 
 html_convert = lambda x: pypandoc.convert_text(x, "html5", "md", ["--mathjax"])
@@ -19,7 +19,6 @@ class LineBuffer:
         self.lines = []
         self.i = 0
         self.insert_next(text)
-        self.onpop = lambda x: None
 
     def insert_next(self, text):
         if isinstance(text, list):
@@ -33,13 +32,20 @@ class LineBuffer:
             raise SyntaxError(f"LineBuffer: unsupported type to insert: {type(text)}")
         self.lines = self.lines[: self.i] + new_lines + self.lines[self.i :]
 
-    def pop(self) -> str:
+    def _pop(self) -> str:
         if self.i == len(self.lines):
             raise SyntaxError("File terminated unexpectedly")
         self.i += 1
-        self.onpop(self)
         return self.lines[self.i - 1]
 
+    def pop(self) -> str:
+        line = self._pop()
+        stripped = line.rstrip()
+        while stripped.endswith("\\"):
+            line = stripped[:-1] + "\n" + self._pop()
+            stripped = line.rstrip()
+        return line
+
     def remove_prev(self):
         self.i -= 1
         if self.i < 0:
@@ -85,10 +91,6 @@ def process_title(line):
         return line.strip(), has_fixed, None
 
 
-def rand_id():
-    return "".join(random.choices(string.ascii_uppercase, k=32))
-
-
 class ToParse:
     def __init__(self, text, type):
         self.text = text
@@ -102,7 +104,9 @@ def parse(text):
     return {"text": text, "html": ToParse(text, "html"), "tex": ToParse(text, "tex")}
 
 
-def parse_define(directive, rest, substitutions, substitutions_match):
+def parse_define(
+    directive, rest, substitutions, substitutions_match, substitution_groups
+):
     if directive == "MATCH":
         regex = r"\[(.*)\]\s+\[(.*)\]"
         matches = re.match(regex, rest)
@@ -118,6 +122,19 @@ def parse_define(directive, rest, substitutions, substitutions_match):
         substitutions_match.append(
             {"directives": directives_list, "replacements": replacements_list}
         )
+    elif directive == "GROUP":
+        regex = r"(\([^()]+\)\s*)+"
+        matches = re.match(regex, rest)
+        if not matches:
+            raise SyntaxError("Invalid declaration of DEFINE GROUP")
+        blocks = re.findall(r"\([^()]+\)", rest)
+        if len(blocks) <= 1:
+            raise SyntaxError("DEFINE GROUP is incomplete")
+        for i, block in enumerate(blocks):
+            blocks[i] = tuple(block[1:-1].split(" "))
+        if not all(len(block) == len(blocks[0]) for block in blocks):
+            raise SyntaxError("DEFINE GROUP blocks must all be of the same length")
+        substitution_groups.append(blocks)
     else:
         substitutions[directive] = rest.split(" ")
 
@@ -205,6 +222,7 @@ def consume_rest_of_question(buff):
     input_lines = []
     substitutions = {}
     substitutions_match = []
+    substitution_groups = []
     solution = None
     solution_note = None
     config = {}
@@ -249,13 +267,20 @@ def consume_rest_of_question(buff):
                     "options": options,
                     "substitutions": substitutions,
                     "substitutions_match": substitutions_match,
+                    "substitution_groups": substitution_groups,
                 }
             else:
                 raise SyntaxError(
                     f"Unexpected END {directive if directive else line} in QUESTION"
                 )
         elif mode == "DEFINE":
-            parse_define(directive, rest, substitutions, substitutions_match)
+            parse_define(
+                directive,
+                rest,
+                substitutions,
+                substitutions_match,
+                substitution_groups,
+            )
         elif mode == "CONFIG":
             config[directive] = rest
         else:
@@ -270,6 +295,7 @@ def consume_rest_of_group(buff, end):
     started_elements = False
     substitutions = {}
     substitutions_match = []
+    substitution_groups = []
     pick_some = None
     scramble = False
     inline = False
@@ -309,12 +335,15 @@ def consume_rest_of_group(buff, end):
                 "elements": elements,
                 "substitutions": substitutions,
                 "substitutions_match": substitutions_match,
+                "substitution_groups": substitution_groups,
                 "pick_some": pick_some,
                 "scramble": scramble,
                 "inline": inline,
             }
         elif mode == "DEFINE":
-            parse_define(directive, rest, substitutions, substitutions_match)
+            parse_define(
+                directive, rest, substitutions, substitutions_match, substitution_groups
+            )
         elif mode == "CONFIG":
             if directive == "PICK":
                 if pick_some:
@@ -342,6 +371,7 @@ def _convert(text, *, path=None):
     config = {}
     substitutions = {}
     substitutions_match = []
+    substitution_groups = []
     try:
         if path is not None:
             handle_imports(buff, path)
@@ -380,7 +410,13 @@ def _convert(text, *, path=None):
                 else:
                     groups.append(group)
             elif mode == "DEFINE":
-                parse_define(directive, rest, substitutions, substitutions_match)
+                parse_define(
+                    directive,
+                    rest,
+                    substitutions,
+                    substitutions_match,
+                    substitution_groups,
+                )
             else:
                 raise SyntaxError(f"Unexpected directive: {line}")
     except SyntaxError as e:
@@ -394,6 +430,7 @@ def _convert(text, *, path=None):
         "config": config,
         "substitutions": substitutions,
         "substitutions_match": substitutions_match,
+        "substitution_groups": substitution_groups,
         "version": VERSION,
     }
 
