@@ -15,7 +15,9 @@ from examtool.api.gradescope_upload import APIClient
 @click.option("--course", prompt=True, help="The Gradescope course ID.")
 @click.option("--assignment", prompt=True, help="The Gradescope assignment ID.")
 @click.option("--email", prompt=True, help="Your Gradescope email address.")
-@click.option("--password", prompt=True, hide_input=True, help="Your Gradescope account password.")
+@click.option(
+    "--password", prompt=True, hide_input=True, help="Your Gradescope account password."
+)
 @exam_name_option
 @hidden_target_folder_option
 def gradescope_upload(course, assignment, email, password, exam, target):
@@ -44,8 +46,10 @@ def gradescope_upload(course, assignment, email, password, exam, target):
             continue
         student_email = file_name[:-4]
         # print("Uploading:", file_name)
-        client.upload_submission(course, assignment, student_email, os.path.join(target, file_name))
+        client.upload_submission(
+            course, assignment, student_email, os.path.join(target, file_name)
+        )
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     gradescope_upload()
